import gensim
import numpy as np
import os
import matplotlib.pyplot as plt

SIZE = 20
plt.rc('font', size=SIZE)  # controls default text sizes
plt.rc('axes', titlesize=SIZE)  # fontsize of the axes title
plt.rc('axes', labelsize=SIZE)  # fontsize of the x any y labels
plt.rc('xtick', labelsize=23)  # fontsize of the tick labels
plt.rc('ytick', labelsize=SIZE)  # fontsize of the tick labels
plt.rc('legend', fontsize=SIZE)  # legend fontsize
plt.rc('figure', titlesize=SIZE)


def softmax(x):
    return np.exp(x) / np.exp(x).sum(axis=0)


def score(model, testing, training, neg, k):
    word2id = model.word2id
    N = len(training)
    initial_prob = np.zeros(k)
    for it in xrange(N):
        x = training[it]
        initial_prob[word2id[x[0]]] += 1
    initial_prob /= float(np.sum(initial_prob))

    V = model.syn0
    if neg == 20:
        U = model.syn1neg
    else:
        U = model.syn1

    N = len(testing)
    costs = 0
    for it in xrange(N):
        x = testing[it]
        n = len(x)
        for jj in xrange(0, n):
            if jj > 0:
                current = word2id[x[jj]]
                previous = word2id[x[jj - 1]]
                z = np.dot(V[previous, :], U.T)
                y = softmax(z)
                costs += np.log(y[current])
            else:
                if initial_prob[word2id[x[0]]] > 0:
                    costs += np.log(initial_prob[word2id[x[0]]])
                else:
                    costs += np.log(1e-6)

    return -(costs / float(N))


def run_w2v(text, training, dim, iteration, windows, hs, sg, neg):
    model = gensim.models.Word2Vec(size=dim, iter=iteration, window=windows, workers=10,
                                   negative=neg, min_count=0, hs=hs, sg=sg,
                                   alpha=1e-3, min_alpha=1e-3, seed=3)
    model.build_vocab(text)

    gap = (1e-3 - 1e-6) / 30
    prob = np.zeros(30)
    for i in xrange(30):
        model.train(training)
        model.alpha -= gap
        model.word2id = dict((w, v.index) for w, v in model.vocab.iteritems())
        prob[i] = score(model, training, training, neg, 40)

    model.word2id = dict((w, v.index) for w, v in model.vocab.iteritems())
    model.word_vectors = model.syn0norm

    plt.plot(prob, linewidth=5)
    plt.ylabel('-LL')
    plt.xlabel('Number of epochs')
    plt.show()

    return model


def run_optimize(cv, text, items, parameters):
    i = 0
    results = np.zeros([2, 4])
    for train_valid in [1]:
        if cv != train_valid:
            training = [text[j] for j, val in enumerate(items) if
                        val != train_valid and val != cv]
            testing = [text[j] for j, val in enumerate(items) if
                       val == train_valid and val != cv]
            ii = 0
            for hs in xrange(2):
                if hs:
                    neg = 0
                else:
                    neg = 20
                for sg in xrange(2):
                    model = run_w2v(text, training, parameters[ii][1], parameters[ii][2], parameters[ii][3],
                                    hs, sg, neg)

                    results[i, ii] = score(model, testing, training, neg, parameters[ii][0])
                    ii += 1
            i += 1
    return np.mean(results, axis=0)


def run_test(cv, text, items, parameters):
    i = 0
    results = np.zeros(4)
    training = [text[j] for j, val in enumerate(items) if
                val != cv]
    testing = [text[j] for j, val in enumerate(items) if
               val == cv]
    ii = 0
    for hs in xrange(2):
        if hs:
            neg = 0
        else:
            neg = 20
        for sg in xrange(2):
            model = run_w2v(text, training, parameters[ii][1], parameters[ii][2], parameters[ii][3],
                            hs, sg, neg)

            results[ii] = score(model, testing, training, neg, parameters[ii][0])
            ii += 1
    return results


def main():
    iteration_arr = xrange(50, 601, 50)
    dim_arr = xrange(5, 41, 5)
    window_arr = xrange(1, 15, 1)
    for file_tmp in os.listdir("./Data/Trajectories"):
        if file_tmp.endswith(".txt"):
            str_file = file_tmp.title()
            name = str_file[:-4]
            items = []
            if name == 'Yuri':
                if os.path.isfile('./Data/Uniques/pre_compute_dtw_%s.npy' % name):
                    with open('./Data/CV/cross_validation_%s.txt' % name, 'r') as f:
                        for line in f:
                            tmp = line.strip('\n')
                            items.append(int(tmp))
                    results_cv = np.zeros([3, 4])
                    for cv in xrange(1):
                        parameters = np.array([[40, 10, 200, 1], [40, 10, 200, 1], [40, 10, 200, 1], [40, 10, 200, 1]])
                        # parameters = np.array([[40, 10, 100, 3], [40, 10, 100, 3], [40, 10, 100, 3], [40, 10, 100, 3]])
                        lst_labels = []
                        if os.path.isfile('./Data/Labels/labels_%s_%d.txt' % (name, parameters[0, 0])):
                            with open('./Data/Labels/labels_%s_%d.txt' % (name, parameters[0, 0]), 'r') as f:
                                for line in f:
                                    tmp = line.strip('\n')
                                    lst_labels.append(tmp)
                            count = 1
                            text = []
                            sentence = []
                            for element in lst_labels:
                                sentence.append(element)
                                if count % 14 == 0:
                                    text.append(sentence)
                                    sentence = []
                                    count = 0
                                count += 1

                        # for i in xrange(3):
                        #     print '%s CV #%d optimize #%d' % (name, cv, i)
                        #     # print '%s CV #%d iteration' % (name, cv)
                        #     results = np.zeros([len(iteration_arr), 4])
                        #     for ii, iteration in enumerate(iteration_arr):
                        #         parameters[:, 2] = iteration
                        #         results[ii, :] = run_optimize(cv, text, items, parameters)
                        #
                        #     results = results[~np.all(results == 0, axis=1)]
                        #     tmp = np.argmin(results, axis=0)
                        #     parameters[0, 2] = iteration_arr[tmp[0]]
                        #     parameters[1, 2] = iteration_arr[tmp[1]]
                        #     parameters[2, 2] = iteration_arr[tmp[2]]
                        #     parameters[3, 2] = iteration_arr[tmp[3]]
                        #
                        #     # print '%s CV #%d dim' % (name, cv)
                        #     results = np.zeros([len(dim_arr), 4])
                        #     for ii, dim in enumerate(dim_arr):
                        #         parameters[:, 1] = dim
                        #         results[ii, :] = run_optimize(cv, text, items, parameters)
                        #
                        #     results = results[~np.all(results == 0, axis=1)]
                        #     tmp = np.argmin(results, axis=0)
                        #     parameters[0, 1] = dim_arr[tmp[0]]
                        #     parameters[1, 1] = dim_arr[tmp[1]]
                        #     parameters[2, 1] = dim_arr[tmp[2]]
                        #     parameters[3, 1] = dim_arr[tmp[3]]
                        #
                        #     # print '%s CV #%d window' % (name, cv)
                        #     results = np.zeros([len(window_arr), 4])
                        #     for ii, window in enumerate(window_arr):
                        #         parameters[:, 3] = window
                        #         results[ii, :] = run_optimize(cv, text, items, parameters)
                        #
                        #     results = results[~np.all(results == 0, axis=1)]
                        #     tmp = np.argmin(results, axis=0)
                        #     parameters[0, 3] = window_arr[tmp[0]]
                        #     parameters[1, 3] = window_arr[tmp[1]]
                        #     parameters[2, 3] = window_arr[tmp[2]]
                        #     parameters[3, 3] = window_arr[tmp[3]]
                        #
                        # print '%s CV #%d test' % (name, cv)
                    results_cv[cv, :] = run_test(cv, text, items, parameters)
                    # np.save('./Data/Results_w2v/parameters_%s_cv%d.npy' % (name, cv), parameters)
                    # print results_cv[cv, :]
                    # print parameters
                # np.save('./Data/Results_w2v/results_%s.npy' % name, results_cv)


if __name__ == '__main__':
    main()